% This function computes the dfgls_k (at a given lag order k)
% statistic used for computing Pretest-GLS estimators in all 
% the figures presenting the simulation results. The function 
% is modified from Senera Ng's original code at 
% https://sites.google.com/site/sn2294/home/code-and-data

function [dfgls] = dfgls_k(y,p,k);

nt = length(y);
if p == 0;
    z = ones(nt,1);
    cbar = -7.0;
end
if p == 1;
    z =[ones(nt,1), [1:nt]'];
    cbar = -13.5;
end

[yt,ssra] = glsd(y,z,cbar); % transforming the data%
dyt = diff(yt);
T = nt-1;
depvar = dyt(k+1:T);
regs = [yt(k+1:T)];
if k > 0
    for i = 1:k
        regs = [regs, dyt(k-i+1:T-i)];
    end
end

[beta, resid, ~] = ols(depvar,regs);
nef = length(dyt);
s2e = resid'*resid/nef;
xx = inv(regs'*regs);
sre = xx(1,1)*s2e;
dfgls = beta(1,1)/sqrt(sre);

end

%%% --- Below are some functions built in the main function dfgls_k.m ---
function [yt,ssr] = glsd(y,z,cbar);
nt = length(y);
abar = 1+cbar/nt;
ya = zeros(nt,1);
za = zeros(nt,length(z(1,:)));
ya(1:1,1) = y(1:1,1);
za(1:1,:) = z(1:1,:);
ya(2:nt,1) = y(2:nt,1)-abar*y(1:nt-1,1);
za(2:nt,:) = z(2:nt,:)-abar*z(1:nt-1,:);
%constructing the gls detrended series%
bhat = inv(za'*za)*za'*ya;
yt = y-z*bhat;
ssr = (ya-za*bhat)'*(ya-za*bhat);
end

